%% Read Me

% dilRheoData imports the .txt output from the microtensiometer and returns
% relevant dilatational rheological parameters, namely E', E", phase angle
% and |E*|, for user-specified regions in the data.  Rheological parameters
% and accompanying figures are saved in a newly created folder that shares
% the name of the text input file.
%
%
close all;clear all;fclose('all');
%
%
%-------------------------------Instructions------------------------------%
%
% 1) In the input file, ensure that the header information is correct and
%    that the data are uninterrupted from start to finish (make sure no
%    header information is buried later on like when you stop/start data
%    collection without changing the file name in LabVIEW).
%
% 2) Specify both the directory (with backslashes) and full
%    name (including .txt) of the input file below.  For example,
%    dirname = 'C:\Users\ArtificialHuman\Folder\' and 
%    filename = 'ExampleFile.txt'.  The directory is easily accessible by
%    copy/paste from Windows Explorer.

%   Import the data from the text file
[filename,dirname] = uigetfile({'*.txt';'*.xls';'*.xlsx';'*.csv'});
%data_primary = importdata(strcat(dirname,filename));

%    USER SPECIFY directory of input file
     %dirname = 'C:\Users\War Machine_MD\Box Sync\Research\Microtens\Analysis\DilRheo\test data\';
     %dirname = '/Users/souravbarman/Google Drive (sbarman@umn.edu)/Work/Data/07_24_19/';
%    USER SPECIFY name of input file
     %filename = 'new code test.txt';
     %filename = 'Lyso 16 1uM slow osc  bubble 1.txt';
%
% 3) Specify the hydrostatic pressure (Pa) to be subtracted from the
%    pressure signal and the surface tension of a clean interface (mN/m).
%
%    USER SPECIFY value of hydrostatic pressure (Pa)
     HSP = 0;
%
%    USER SPECIFY value of clean interface (mN/m)
     clean_IFT = 72.8;
%
% 4) Specify whether additional data files should be generated in addition
%    to the summary file which contains all of the pertinent rheological
%    parameters.
%
%    USER SPECIFY the value of extra.  'extra = 1' will generate additional
%    files, 'extra = 0' will not.
%
     extra = 1;
     %extra = 0;
%
%    NOTE: Additional instructions concern steps that occur after this
%    script is run.  Read the additional steps before running this script,
%    and follow on-screen instructions.
%
% 5)
%
% . . .
%
% 
%
% adapted from code written by A Kotula 13-apr-15, S Kirby 14-aug-21
% created by M Davidson 19-feb-7
% updated by M Davidson 19-jul-25

%% S1 - Variable Definitions (WIP)

% capR -- value of capillary radius read from header of input (um)
% clean_IFT -- surface tension of interface without surfactant (mN/m)
% dirname -- directory name of input file
% error_surfarea -- error propagated to surface area
% error_surf -- error propagated to surface tension
% fig_S3 -- graphical object used in Section 3
% file_input -- concatenated directory and name of input file
% fileID -- file identification returned by fopen()
% filename -- file name of input file
% header -- cell array containing input file header information
% HSP -- value of hydrostatis pressure head input by user (Pa)
% input_c -- value denoting which mouse button was used to click
% input_click -- array holding user input from mouse selections of regions
%                of oscillation
% input_data -- cell array containing data fed to FFT
% input_end -- enables while loop over user mouse input to end on right
%              click
% input_x -- value of time at mouse click
% input_y -- value of pressure at mouse click
% osc_begin -- temporarily stores beginning index of oscillation
% osc_bounds -- indices the mark beginning and end of selected oscillations
% osc_end -- temporarily stores ending index of oscillation
% osc_num -- number of user-selected oscillations
% raw_fit -- vector containing imported fit data
% raw_lag -- vector containing imported time lag data (s)
% raw_pressure -- vector containing imported pressure data (Pa)
% raw_radius -- vector containing imported radius data (um)
% raw_surf -- vector contaning imported surface tension data (mN/m)
% raw_surfHSP -- vector containing surface tension data adjusted for HSP
%                (mN/m)
% raw_time -- vector containing imported time data (s)
% readdata -- data read by textscan()
% readline -- line read by textscan()
% shift_time -- raw_time shifted to begin at 0
% shift_timeR -- shift_time corrected for time lag from radius signal
% sub_pressure -- linear drift subtracted from raw_pressure
% sub_radius = inear drift subtracted from raw_radius
% sub_surfarea = inear drift subtracted from raw_surfarea
% sub_surfHSP = inear drift subtracted from surfHSP

%% S2 - Specifying Input File Location and Importing Data

% concatenate dirname and filename into a new variable for ease
file_input = [dirname filename];

% open text file
fileID = fopen(file_input,'r');
% zero position of text scan
frewind(fileID);
% read header information
for count_header = 1:9
    readline = {fgetl(fileID)};
    header{count_header} = readline;
end
% zero position of text scan
frewind(fileID);
% read capillary radius from text file
readline = textscan(fileID,'%*11c %f',1);

%isolate capR
caprtext=char(header{2});
caprstr=caprtext(24:length(caprtext));

capR = str2double(caprstr);%readline{1};
% zero position of text scan
frewind(fileID);
% read data without header into a cell array
readdata = textscan(fileID,'%f %f %f %f %f %f %f','HeaderLines',11);
% create vectors from the columns in readdata
raw_time = readdata{1}; %time
raw_surf = readdata{2}; %surface tension (without HSP)
raw_pressure = readdata{4}; %pressure; we chose column 4 since it is already hydrostatic pressure corrected
raw_radius = readdata{5}; %radius
raw_fit = readdata{6}; %fit

height_data=size(readdata{6});
% raw_lag_dummy= zeros(height_data);
% raw_lag_dummy=raw_lag_dummy+0.015;
raw_lag = readdata{7}; %lag
% close text file
fclose(fileID);

%% S3 - Determining Regions of Oscillation from User Input

% plot pressure signal to select oscillation regions
fig_S3 = figure('Name','Pressure Signal','NumberTitle','off',...
    'Units','normalized','Position',[0 0 1 1]);

yyaxis left
plot(raw_time,raw_pressure,'k-')% changed from raw_pressure to raw_radius to check the radius data
xlabel('time (s)');ylabel('pressure (Pa)');title('Left click to select regions of oscillations.  Right click when finished.')
yyaxis right
plot(raw_time,raw_surf,'r')
ylabel('Surface Tension, mN/m')

% initialize array to hold user mouse click input
input_click = [];
% value controlling loop over collecting using mouse input
input_end = 0;
% continuously collect user input until a right mouse click
while(input_end < 1)
    % collect user input
    [input_x,input_y,input_c] = ginput(1);
    input_click = [input_click; input_x input_y input_c];
    % break loop on right click
    if input_click(end,3)==3
        input_end = 1;
    end
end
% abort on single right click
if input_click(1,3)==3
    return
end
% abort if input_click has an even number of rows, indicating user error
if mod(length(input_click),2)==0
    close all
    msgbox({'Invalid Selection','Select bounds of oscillations with left click.  Right click when finished.'},'Input Error','error');
    return
end
% number of selected oscillations
osc_num = (length(input_click)-1)/2;
% determine indices that correspond to selected times
osc_bounds = zeros(osc_num,2);
for count_osc = 1:osc_num
    % osc start index
    osc_begin = input_click(2*count_osc-1,1);
    osc_bounds(count_osc,1) = find(raw_time>osc_begin,1);
    % osc end index
    osc_end = input_click(2*count_osc,1);
    osc_bounds(count_osc,2) = find(raw_time>osc_end,1);
end

%% S4 - Pretreat Data for FFT
close all
% shift time to zero initial point
shift_time = raw_time - raw_time(1);
% correct for delay time in Radius signal
shift_timeR = shift_time + raw_lag;
% calculate surface area at each time point
raw_surfarea = 2*pi.*raw_radius.*(raw_radius-sqrt(raw_radius.^2-capR^2));
% calculate surface tension considering HSP
raw_surfHSP = (raw_pressure-HSP).*raw_radius/2000;
% calculate error on surface area
error_surfarea = raw_surfarea.*abs(raw_fit.*(1./raw_radius-1./sqrt(raw_radius.^2-capR^2)));
error_surf = raw_surf.*sqrt((10./raw_pressure).^2+(raw_fit./raw_radius).^2);
% create a cell array to hold data from selected oscillations
input_data = cell(osc_num,1);
% create an array to hold rheology data from selected oscillations
rheo_data = NaN(19,osc_num);
% place data into the cell array
Lissajou = figure;
for count_cell = 1:osc_num
    % oscillation bounds
    osc_begin = osc_bounds(count_cell,1);
    osc_end = osc_bounds(count_cell,2);
    % store steady state values and errors
    rheo_data(11,count_cell) = mean(raw_pressure(osc_begin:osc_end)-HSP);
    rheo_data(12,count_cell) = 10;
    rheo_data(13,count_cell) = mean(raw_radius(osc_begin:osc_end));
    rheo_data(14,count_cell) = mean(raw_fit(osc_begin:osc_end));
    rheo_data(15,count_cell) = clean_IFT - mean(raw_surfHSP(osc_begin:osc_end));
    rheo_data(16,count_cell) = mean(error_surf);
    rheo_data(17,count_cell) = mean(raw_surfarea(osc_begin:osc_end));
    rheo_data(18,count_cell) = mean(error_surfarea(osc_begin:osc_end));
    
    % subtract linear drift
    sub_pressure = detrend(raw_pressure(osc_begin:osc_end),1);
    sub_radius = detrend(raw_radius(osc_begin:osc_end),1);
    sub_surfarea = detrend(raw_surfarea(osc_begin:osc_end),1);
    sub_surfHSP = detrend(raw_surfHSP(osc_begin:osc_end),1);
    input_data{count_cell} = [shift_time(osc_begin:osc_end), ...
                              raw_surfHSP(osc_begin:osc_end), ...
                              error_surf(osc_begin:osc_end), ...
                              sub_surfHSP, ...
                              raw_pressure(osc_begin:osc_end)-HSP, ...
                              sub_pressure, ...
                              raw_radius(osc_begin:osc_end), ...
                              raw_fit(osc_begin:osc_end), ...
                              sub_radius, ...
                              raw_lag(osc_begin:osc_end), ...
                              shift_timeR(osc_begin:osc_end), ...
                              raw_surfarea(osc_begin:osc_end), ...
                              error_surfarea(osc_begin:osc_end), ...
                              sub_surfarea];
                          
                          
    %figure('Name','Lissajou plots','NumberTitle','off');
    %Lissajou = figure;
    subplot(1,osc_num,count_cell);
    plot(sub_surfarea,sub_surfHSP,'b-');
    xlabel('dlnA');ylabel('d\gamma');
    hold on
    grid on
    
end

% saveas(Lissajou, filename, 'epsc');

%% S5 - Perform FFT on Selected Oscillations.  Plot results

%close all

% construct a cell array to fft vector outputs (1st column has frequency
% domain data, 2nd column has time domain data)
fft_vector = cell(osc_num,2);
% construct matrices to store fft scalar outputs (1 values, 1 errors)
fft_scalar = NaN(12,osc_num);
fft_scalarErr = fft_scalar;

% looping through fft
for count_fft = 1:osc_num
    % retrieve fft data from input_data
    fft_time = input_data{count_fft}(:,1)-input_data{count_fft}(1,1);
    fft_pressure = input_data{count_fft}(:,6);
    fft_surfHSP = input_data{count_fft}(:,4);
    fft_STErr = input_data{count_fft}(:,3);
    fft_timeR = input_data{count_fft}(:,11)-input_data{count_fft}(1,11);
    fft_fit = input_data{count_fft}(:,8);
    fft_radius = input_data{count_fft}(:,9);
    fft_surfarea = input_data{count_fft}(:,14);
    fft_SAErr = input_data{count_fft}(:,13);

    % determine principle frequency using fastlomb()
    [lomb_P,lomb_w,lomb_a] = fastlomb(fft_pressure,fft_time);
    lomb_wprince = lomb_w(lomb_P==max(lomb_P));
    % check that data possesses at least 6 periods at the principle frequency
    lomb_tprince = 1/lomb_wprince;  %requires only 1 period
    %lomb_tprince = 6/lomb_wprince;
    if lomb_tprince > fft_time(end)
        close all
        msgbox({'More periods of oscillation required.','Select at least 2 complete periods for each frequency'},'Selection Error','error');
        return
    end
    % determine a sampling interval from the time spacing of the input data
    % (this will be the same as the camera's frame rate)
    fft_sampling = floor(numel(find(fft_time<10))/10);
    % construct an evenly spaced time vector
    fft_Teven = 0:1/fft_sampling:fft_time(end)-mod(fft_time(end),1/fft_sampling);
    fft_TevenR = 0:1/fft_sampling:fft_timeR(end)-mod(fft_timeR(end),1/fft_sampling);
    % use a linear interpolater to evenly space signals
    fft_Peven = interp1(fft_time,fft_pressure,fft_Teven(2:end),'linear')';
    fft_STeven = interp1(fft_time,fft_surfHSP,fft_Teven(2:end),'linear')';
    fft_Reven = interp1(fft_timeR,fft_radius,fft_TevenR(2:end),'linear')';
    fft_SAeven = interp1(fft_timeR,fft_surfarea,fft_TevenR(2:end),'linear')';

    % determine extent to pad data to speed up FFT
    fft_pad = 2^(nextpow2(2*length(fft_Peven))+6);
    % determine frequencies interrogated by FFT
    fft_freqs = fft_sampling/2*linspace(0,1,fft_pad/2+1)';
    % perform FFT
    fft_Pcoefs = fft(fft_Peven,fft_pad)/length(fft_Peven);fft_Pcoefs = fft_Pcoefs(1:fft_pad/2+1);
    fft_PcoefsABS = abs(fft_Pcoefs);
    fft_STcoefs = fft(fft_STeven,fft_pad)/length(fft_STeven);fft_STcoefs = fft_STcoefs(1:fft_pad/2+1);
    fft_STcoefsABS = abs(fft_STcoefs);
    fft_Rcoefs = fft(fft_Reven,fft_pad)/length(fft_Reven);fft_Rcoefs = fft_Rcoefs(1:fft_pad/2+1);
    fft_RcoefsABS = abs(fft_Rcoefs);
    fft_SAcoefs = fft(fft_SAeven,fft_pad)/length(fft_SAeven);fft_SAcoefs = fft_SAcoefs(1:fft_pad/2+1);
    fft_SAcoefsABS = abs(fft_SAcoefs);
    % determine frequency of principle harmonic (modify here to find other
    % harmonics)
    [~,fft_Pindices] = sort(fft_PcoefsABS,'descend');
    fft_Pfreq = fft_freqs(fft_Pindices(1));
    [~,fft_STindices] = sort(fft_STcoefsABS,'descend');
    fft_STfreq = fft_freqs(fft_STindices(1));
    [~,fft_Rindices] = sort(fft_RcoefsABS,'descend');
    fft_Rfreq = fft_freqs(fft_Rindices(1));
    [~,fft_SAindices] = sort(fft_SAcoefsABS,'descend');
    fft_SAfreq = fft_freqs(fft_SAindices(1));
    % determine phase angle of the principle harmonic
    fft_Pangles = angle(fft_Pcoefs)+pi/2;
    fft_Pphase = fft_Pangles(fft_freqs==fft_Pfreq);
    fft_STangles = angle(fft_STcoefs)+pi/2;
    fft_STphase = fft_STangles(fft_freqs==fft_STfreq);
    fft_Rangles = angle(fft_Rcoefs)+pi/2;
    fft_Rphase = fft_Rangles(fft_freqs==fft_Rfreq);
    fft_SAangles = angle(fft_SAcoefs)+pi/2;
    fft_SAphase = fft_SAangles(fft_freqs==fft_SAfreq);
    % determine amplitude of the principle harmonic
    fft_Pamp = fft_PcoefsABS(fft_freqs==fft_Pfreq)*2;
    fft_STamp = fft_STcoefsABS(fft_freqs==fft_STfreq)*2;
    fft_Ramp = fft_RcoefsABS(fft_freqs==fft_Rfreq)*2;
    fft_SAamp = fft_SAcoefsABS(fft_freqs==fft_SAfreq)*2;

    % determine errors from FFT using FWHM calculation assuming Gaussian noise
    error_freq = mean(diff(fft_freqs));
    %pressure
    fft_Preal = real(fft_Pcoefs(fft_freqs==fft_Pfreq));
    fft_Pimag = imag(fft_Pcoefs(fft_freqs==fft_Pfreq));
    fft_Pmag = sqrt(fft_Preal^2+fft_Pimag^2);
    error_Preal = sqrt(sum((cos(2*pi*fft_Pfreq.*fft_time).*10).^2))/length(fft_pressure);
    error_Pimag = sqrt(sum((sin(2*pi*fft_Pfreq.*fft_time).*10).^2))/length(fft_pressure);
    error_Pmag = -1/2*sum(mean(diff(fft_time))^2*sin(2*(2*pi*fft_Pfreq).*fft_time).*10.^2)/length(fft_pressure);
    error_Pphase = sqrt((fft_Pimag*error_Preal)^2+(fft_Preal*error_Pimag)^2-2*fft_Preal*fft_Pimag*error_Pmag^2)/fft_Pmag^2;
    error_Pamp = 2/fft_Pmag*sqrt((fft_Preal*error_Preal)^2+(fft_Pimag*error_Pimag)^2+2*(fft_Preal*fft_Pimag*error_Pmag^2));
    %surface tension
    fft_STreal = real(fft_STcoefs(fft_freqs==fft_STfreq));
    fft_STimag = imag(fft_STcoefs(fft_freqs==fft_STfreq));
    fft_STmag = sqrt(fft_STreal^2+fft_STimag^2);
    error_STreal = sqrt(sum((cos(2*pi*fft_STfreq.*fft_time).*mean(fft_STErr)).^2))/length(fft_surfHSP);
    error_STimag = sqrt(sum((sin(2*pi*fft_STfreq.*fft_time).*mean(fft_STErr)).^2))/length(fft_surfHSP);
    error_STmag = -1/2*sum(mean(diff(fft_time))^2*sin(2*(2*pi*fft_STfreq).*fft_time).*mean(fft_STErr).^2)/length(fft_surfHSP);
    error_STphase = sqrt((fft_STimag*error_STreal)^2+(fft_STreal*error_STimag)^2-2*fft_STreal*fft_STimag*error_STmag^2)/fft_STmag^2;
    error_STamp = 2/fft_STmag*sqrt((fft_STreal*error_STreal)^2+(fft_STimag*error_STimag)^2+2*(fft_STreal*fft_STimag*error_STmag^2));
    %radius
    fft_Rreal = real(fft_Rcoefs(fft_freqs==fft_Rfreq));
    fft_Rimag = imag(fft_Rcoefs(fft_freqs==fft_Rfreq));
    fft_Rmag = sqrt(fft_Rreal^2+fft_Rimag^2);
    error_Rreal = sqrt(sum((cos(2*pi*fft_Rfreq.*fft_timeR).*mean(fft_fit)).^2))/length(fft_radius);
    error_Rimag = sqrt(sum((sin(2*pi*fft_Rfreq.*fft_timeR).*mean(fft_fit)).^2))/length(fft_radius);
    error_Rmag = -1/2*sum(mean(diff(fft_timeR))^2*sin(2*(2*pi*fft_Rfreq).*fft_timeR).*mean(fft_fit).^2)/length(fft_radius);
    error_Rphase = sqrt((fft_Rimag*error_Rreal)^2+(fft_Rreal*error_Rimag)^2-2*fft_Rreal*fft_Rimag*error_Rmag^2)/fft_Rmag^2;
    error_Ramp = 2/fft_Rmag*sqrt((fft_Rreal*error_Rreal)^2+(fft_Rimag*error_Rimag)^2+2*(fft_Rreal*fft_Rimag*error_Rmag^2));
    %surface area
    fft_SAreal = real(fft_SAcoefs(fft_freqs==fft_SAfreq));
    fft_SAimag = imag(fft_SAcoefs(fft_freqs==fft_SAfreq));
    fft_SAmag = sqrt(fft_SAreal^2+fft_SAimag^2);
    error_SAreal = sqrt(sum((cos(2*pi*fft_SAfreq.*fft_timeR).*mean(fft_SAErr)).^2))/length(fft_surfarea);
    error_SAimag = sqrt(sum((sin(2*pi*fft_SAfreq.*fft_timeR).*mean(fft_SAErr)).^2))/length(fft_surfarea);
    error_SAmag = -1/2*sum(mean(diff(fft_timeR))^2*sin(2*(2*pi*fft_SAfreq).*fft_timeR).*mean(fft_SAErr).^2)/length(fft_surfarea);
    error_SAphase = sqrt((fft_SAimag*error_SAreal)^2+(fft_SAreal*error_SAimag)^2-2*fft_SAreal*fft_SAimag*error_SAmag^2)/fft_SAmag^2;
    error_SAamp = 2/fft_SAmag*sqrt((fft_SAreal*error_SAreal)^2+(fft_SAimag*error_SAimag)^2+2*(fft_SAreal*fft_SAimag*error_SAmag^2));
    
    % retrieve steady state values
    ss_P = rheo_data(11,count_fft);
    ss_Perr = rheo_data(12,count_fft);
    ss_R = rheo_data(13,count_fft);
    ss_Rerr = rheo_data(14,count_fft);
    ss_SP = rheo_data(15,count_fft);
    ss_ST = clean_IFT - ss_SP;
    ss_SPerr = rheo_data(16,count_fft);
    ss_SA = rheo_data(17,count_fft);
    ss_SAerr = rheo_data(18,count_fft);
    
    % calculate rheological parameters from APK-SLA JoR paper
    APK_eps = fft_Pamp/ss_P;
    APK_dR = fft_Ramp/(APK_eps*ss_R);
    APK_b1 = sqrt(1-(capR/ss_R)^2);
    APK_dphi = fft_Rphase - fft_Pphase;
    
    Area_strain_percentage = (fft_SAamp/ss_SA)*100;
    
    APK_Emag = APK_b1/(1-APK_b1)*sqrt(APK_dR^2+2*APK_dR*cos(APK_dphi)+1)/APK_dR*ss_ST;
    %APK_Ephi = (atan2(sin(APK_dphi),cos(APK_dphi))-atan2(APK_dR*sin(APK_dphi),1+APK_dR*cos(APK_dphi)));
    %APK_Ephi = pi - (atan2(sin(APK_dphi),cos(APK_dphi))-atan2(APK_dR*sin(APK_dphi),1+APK_dR*cos(APK_dphi)));

    
    APK_Ephi = (atan2(sin(APK_dphi),cos(APK_dphi))-atan2(APK_dR*sin(APK_dphi),1+APK_dR*cos(APK_dphi)));
    if APK_Ephi <0 || APK_Ephi >pi/2
        APK_Ephi = pi+(atan2(sin(APK_dphi),cos(APK_dphi))-atan2(APK_dR*sin(APK_dphi),1+APK_dR*cos(APK_dphi)));
    end

    APK_Ereal = abs(APK_Emag*cos(APK_Ephi));
    APK_Eimag = abs(APK_Emag*sin(APK_Ephi));
    
    % calculate uncertainty in rheo parameters
    APK_epserr = APK_eps*ss_Perr/ss_P;
    APK_dRerr = APK_dR*sqrt((ss_Rerr/ss_R)^2+(APK_epserr/APK_eps)^2);
    APK_b1err = APK_b1*sqrt((0.1/capR)^2+(ss_Rerr/ss_R)^2);
    APK_dphierr = sqrt(error_Rphase^2+error_Pphase^2);
    
    APK_Emagerr = APK_Emag*sqrt((APK_b1err/APK_b1)^2+(APK_dRerr/APK_dR)^2+(APK_dphierr/APK_dphi)^2);
    APK_Ephierr = sqrt(APK_dphierr^2*((1+APK_dR*cos(APK_dphi))/(1+APK_dR^2+2*APK_dR*cos(APK_dphi)))^2+APK_dRerr^2*(sin(APK_dphi)/(1+APK_dR^2+2*APK_dR*cos(APK_dphi)))^2);

    APK_Erealerr = APK_Ereal*sqrt((APK_Emagerr/APK_Emag)^2+(APK_Ephierr*tan(APK_Ephi))^2);
    APK_Eimagerr = APK_Eimag*sqrt((APK_Emagerr/APK_Emag)^2+(APK_Ephierr/tan(APK_Ephi))^2);
    
    % store rheological parameters and uncertainties
    rheo_data(1,count_fft) = fft_Pfreq*2*pi();
    rheo_data(2,count_fft) = error_freq*2*pi();
    rheo_data(3,count_fft) = APK_Emag;
    rheo_data(4,count_fft) = APK_Emagerr;
    rheo_data(5,count_fft) = APK_Ephi;
    rheo_data(6,count_fft) = APK_Ephierr;
    rheo_data(7,count_fft) = APK_Ereal;
    rheo_data(8,count_fft) = APK_Erealerr;
    rheo_data(9,count_fft) = APK_Eimag;
    rheo_data(10,count_fft) = APK_Eimagerr;
    rheo_data(19,count_fft) = Area_strain_percentage;
    
    % store FFT scalars
    fft_scalar(1,count_fft) = fft_Pfreq;
    fft_scalar(2,count_fft) = fft_Pamp;
    fft_scalar(3,count_fft) = fft_Pphase;
    fft_scalar(4,count_fft) = fft_Rfreq;
    fft_scalar(5,count_fft) = fft_Ramp;
    fft_scalar(6,count_fft) = fft_Rphase;
    fft_scalar(7,count_fft) = fft_STfreq;
    fft_scalar(8,count_fft) = fft_STamp;
    fft_scalar(9,count_fft) = fft_STphase;
    fft_scalar(10,count_fft) = fft_SAfreq;
    fft_scalar(11,count_fft) = fft_SAamp;
    fft_scalar(12,count_fft) = fft_SAphase;
    
    % store FFT scalar errors
    fft_scalarErr(1,count_fft) = error_freq;
    fft_scalarErr(2,count_fft) = error_Pamp;
    fft_scalarErr(3,count_fft) = error_Pphase;
    fft_scalarErr(4,count_fft) = error_freq;
    fft_scalarErr(5,count_fft) = error_Ramp;
    fft_scalarErr(6,count_fft) = error_Rphase;
    fft_scalarErr(7,count_fft) = error_freq;
    fft_scalarErr(8,count_fft) = error_STamp;
    fft_scalarErr(9,count_fft) = error_STphase;
    fft_scalarErr(10,count_fft) = error_freq;
    fft_scalarErr(11,count_fft) = error_SAamp;
    fft_scalarErr(12,count_fft) = error_SAphase;
    
    % store FFT vectors (frequency domain)
    fft_vector{count_fft,1} = [fft_freqs, ...
                               fft_PcoefsABS, ...
                               fft_RcoefsABS, ...
                               fft_STcoefsABS, ...
                               fft_SAcoefsABS];
                           
    % store FFT vectors (time domain)
    fft_vector{count_fft,2} = [fft_time, ...
                               fft_pressure, ...
                               fft_radius, ...
                               fft_surfHSP, ...
                               fft_surfarea, ...
                               fft_timeR];
    
    % generate time-domain fit to data from FFT parameters
    fit_pressure = fft_Pamp*sin(2*pi*fft_Pfreq*fft_time+fft_Pphase);
    fit_surfHSP = fft_STamp*sin(2*pi*fft_STfreq*fft_time+fft_STphase);
    fit_radius = fft_Ramp*sin(2*pi*fft_Rfreq*fft_timeR+fft_Rphase);
    fit_surfarea = fft_SAamp*sin(2*pi*fft_SAfreq*fft_timeR+fft_SAphase);

    % plot oscillation data with fit from FFT
    fig_fft(count_fft) = figure('Name',sprintf('FFT at %.4f rad_sec %.2f %% area strain',fft_scalar(1,count_fft)*2*pi(),rheo_data(19,count_fft)),...
        'NumberTitle','off','Units','normalized','Position',[0.02+count_fft*0.03 0.24-count_fft*0.03 0.8 0.7]);
    %pressure
    subplot(2,4,1)
    plot(fft_time,fft_pressure,'ko')
    hold on
    plot(fft_time,fit_pressure,'b-')
    hold off
    xlabel('time (s)');ylabel('pressure (Pa)');title('Pressure')
    %radius
    subplot(2,4,2)
    plot(fft_timeR,fft_radius,'ko')
    hold on
    plot(fft_timeR,fit_radius,'b-')
    hold off
    xlabel('time (s)');ylabel('radius (um)');title('Radius')
    %surface area
    subplot(2,4,3)
    plot(fft_timeR,fft_surfarea,'ko')
    hold on
    plot(fft_timeR,fit_surfarea,'b-')
    hold off
    xlabel('time (s)');ylabel('surface area (um^2)');title('Surface Area')
    %surface tension
    subplot(2,4,4)
    plot(fft_time,fft_surfHSP,'ko')
    hold on
    plot(fft_time,fit_surfHSP,'b-')
    hold off
    xlabel('time (s)');ylabel('surface tension (mN/m)');title('Surface Tension')
    
    % plot power spectra from FFT
    %pressure
    subplot(2,4,5)
    plot(fft_freqs/fft_Pfreq,2*fft_PcoefsABS/fft_Pamp,'k.','MarkerSize',4)
    hold on
    plot([0 20],[0.1 0.1],'b--')
    hold off
    xlim([0 20]);ylim([0 1.1])
    xlabel('Normalized frequency');ylabel('Normalized power')
    %radius
    subplot(2,4,6)
    plot(fft_freqs/fft_Rfreq,2*fft_RcoefsABS/fft_Ramp,'k.')
    hold on
    plot([0 20],[0.1 0.1],'b--')
    hold off
    xlim([0 20]);ylim([0 1.1])
    xlabel('Normalized frequency');ylabel('Normalized power')
    %surface area
    subplot(2,4,7)
    plot(fft_freqs/fft_SAfreq,2*fft_SAcoefsABS/fft_SAamp,'k.')
    hold on
    plot([0 20],[0.1 0.1],'b--')
    hold off
    xlim([0 20]);ylim([0 1.1])
    xlabel('Normalized frequency');ylabel('Normalized power')
    %surface tension
    subplot(2,4,8)
    plot(fft_freqs/fft_STfreq,2*fft_STcoefsABS/fft_STamp,'k.')
    hold on
    plot([0 20],[0.1 0.1],'b--')
    hold off
    xlim([0 20]);ylim([0 1.1])
    xlabel('Normalized frequency');ylabel('Normalized power')
    
end

%% S6 - Export data as .txt files

% make a folder to store data ouput

% name output file for rheo data
rheo_outN = [filename(1:end-4)];

% create labels for rheo_data
rheo_labels = {'f (rad/s)'
    'err(f) (rad/s'
    '|E*| (mN/m)'
    'err(|E*|) (mN/m)'
    'phi (rad)'
    'err(phi) (rad)'
    'E'' (mN/m)'
    'err(E'')(mN/m)'
    'E" (mN/m)'
    'err(E") (mN/m)'
    'P (Pa)'
    'err(P) (Pa)'
    'R (um)'
    'err(R) (um)'
    'surf Pressure (mN/m)'
    'err(surf P) (mN/m)'
    'A (um^2)'
    'err(A) (um^2)'
    'Area Strain (%)'};
% convert rheo_data to cell array and combine with labels;


rheo_cell = [rheo_labels num2cell(rheo_data)];
% write rheo_cell to a text file with name rheo_outN
writecell(rheo_cell,[dirname rheo_outN '.xlsx']);

%create a master summary file

%create a master summary file FOR 2 numbers in the back
if isfile([dirname filename(1:end-7) '-summary.xlsx'])==0
    writecell(rheo_cell, [dirname filename(1:end-7) '-summary.xlsx']);
else
    [num,txt,raw]=xlsread([dirname filename(1:end-7) '-summary.xlsx']);
    appended=[raw num2cell(rheo_data)];
    xlswrite([dirname filename(1:end-7) '-summary.xlsx'], appended);
end
%error because existing data is only 16 row long, so cannot append (the
%process is it reads existing summary file and append with new data, but
%existing data doesnt have information on barrier position. 


% save FFT figures (fig_fft)
for count_fig = 1:osc_num
    % select a figure
    fig_active = fig_fft(count_fig);
    % select a file name and export
    fig_outN = [rheo_outN '.png'];
    saveas(fig_active,[dirname fig_outN]);
end
